@@ -1,25 +1,28 @@
 # -*- coding: utf-8 -*-
 
-from urlparse import urlsplit
 from threading import Thread
 from twisted.internet import reactor, ssl
 from streamer import Streamer
 from handshaker import Handshaker
 from framer import Framer
 from messager import Messager
+from compat import urlsplit
+from exc import *
 
 class WebPocket(object):
-  def __init__(self, url, run=False):
+  def __init__(self, url, run=False, **options):
     self.scheme = None
     self.host = None
     self.port = None
     self.path = None
     self._parse_url(url)
 
+    payload_limit = options.get('payload_limit') or 0
+
     actors = {}
     actors['pocket'] = self
     actors['messager'] = Messager(actors)
-    actors['framer'] = Framer(actors)
+    actors['framer'] = Framer(actors, payload_limit=payload_limit)
     actors['handshaker'] = Handshaker(actors, self.host, self.port, self.path)
     streamer = Streamer(actors)
     actors['streamer'] = streamer
@@ -29,6 +32,8 @@ class WebPocket(object):
       reactor.connectTCP(self.host, self.port, streamer)
     elif self.scheme == 'wss':
       reactor.connectSSL(self.host, self.port, streamer, ssl.ClientContextFactory())
+    else:
+      raise WebSocketProtocolError('Invalid scheme: ' + self.scheme)
 
     self._thread = None
     if run:
@@ -48,6 +53,7 @@ class WebPocket(object):
     self._thread.start()
 
   def stop(self):
+    reactor.callFromThread(self._actors['streamer'].close)
     reactor.callFromThread(reactor.stop)
     self.on_close(1001)
 
